
#ifndef KDTREESCENE_H
#define KDTREESCENE_H

#include <vector>
#include "Scene.h"


// this isn't even needed unless I start using the most significant
// bits of the attr member of KDTreeNode
#define KDTREE_NODE_ATTR_MASK  0x03


class KDTreeScene : public Scene {
public:
  KDTreeScene();
  virtual ~KDTreeScene();

  virtual void insertObject(SceneObject *);

  virtual void insertLight(const Light &light);

  virtual SceneObject *findNearestHit(const Ray &ray,
                                      HitInfo &hitInfoOut);
                                      
  virtual bool occlusionTest(const Ray &ray, float maxT) const;
                                      
  virtual RGBColor<float> lightContrib(const Ray &ray,
                                       const Point3 &pos,
                                       const Vector3 &n,
                                       int matID,
                                       const Point2 &tc);
                                       
  virtual const std::vector<Light>& getLights() const
     { return lights; } // TODO: get rid of this
     
  virtual void build();
  void clear();

  class BoxEdge {
  public:
    BoxEdge() { }
    BoxEdge(float _pos, size_t _objIndex, bool _start);
    bool operator < (const BoxEdge &e) const;
    float pos;
    size_t objIndex;
    bool start;
  };

  class KDTreeNode {
  public:
    KDTreeNode(const AABB &nodeAABB,
               const std::vector<SceneObject *> &sceneObjects,
               size_t maxLeafPrims,
               const std::vector<size_t> &objectsInNode,
               const std::vector<AABB> &objectAABBs,
               int badRefines,
               int depth);
    virtual ~KDTreeNode();

    struct InternalNodeData {
      float planeOffset;
      KDTreeNode *left,
                 *right;
    };

    struct LeafNodeData {
      int numObjects;
      SceneObject **objects;
    };

    inline bool isLeaf() { return (attrs & KDTREE_NODE_ATTR_MASK) == 0x03; }

    inline       unsigned char     getAttrs()            const { return attrs;            }
    inline const InternalNodeData &getInternalNodeData() const { return internalNodeData; }
    inline const LeafNodeData     &getLeafNodeData()     const { return leafNodeData;     }
  private:
    void createLeafNode(const std::vector<SceneObject *> &sceneObjects,
                        const std::vector<size_t> &objectsInNode);
    void createInternalNode(const AABB &nodeAABB,
                            const std::vector<SceneObject *> &sceneObjects,
                            const std::vector<size_t> &objectsInNode,
                            const std::vector<AABB> &objectAABBs,
                            size_t maxLeafPrims,
                            int badRefines,
                            int depth);
    // TODO: make this smaller (preferably less than 8 bytes)
    // flags: bits 0-2 - axis of split (for internal nodes), both are set
    //                   for leaf nodes
    unsigned char attrs;

    union {
      InternalNodeData internalNodeData; // offset of split and child node pointers (for internal nodes)
      LeafNodeData leafNodeData;         // object data (for leaf nodes)
    };
  };

  static const float intersectionCost;
  static const float traversalCost;
  static const float emptyBonus;
  
  static const int maxPrimsInLeaf;
  static const int maxBadRefines;

private:
  static AABB calcAABB(std::vector<SceneObject *> &objs);

  AABB aabb;
  KDTreeNode *root;
  std::vector<SceneObject *> objects;
  std::vector<Light> lights;
};


#endif
