
#include <cfloat>
#include "ListScene.h"


ListScene::~ListScene() {
  for ( size_t i=0; i < sceneObjects.size(); i++ )
    if ( sceneObjects[i] != NULL )
      delete sceneObjects[i];
} // ListScene::~ListScene()


void ListScene::insertObject(SceneObject *obj) {
  if ( NULL == obj )
    return;

  sceneObjects.push_back(obj);
} // ListScene::insertObject()


void ListScene::insertLight(const Light &light) {
  lights.push_back(light);
} // ListScene::insertLight()


RGBColor<float> ListScene::lightContrib(const Ray &ray,
                                        const Point3 &p,
                                        const Vector3 &n,
                                        int matID,
                                        const Point2 &tc) {
  RGBColor<float> color(0.0f, 0.0f, 0.0f);
  RGBColor<float> texSampleColor(1.0f, 1.0f, 1.0f);
  const Material &material = materials[matID];

  // if this material has a texture, sample from it
  if ( material.texture != NULL )
    texSampleColor = material.texture->sample(tc[0], tc[1]);

  // add contribution from each light
  for ( size_t i=0; i < lights.size(); i++ )
    color += singleLightContrib(lights[i], p, ray, n, material);

  return texSampleColor*color;
} // ListScene::lightContrib()


SceneObject *ListScene::findNearestHit(const Ray &ray,
                                       HitInfo &hitInfoOut) {
  SceneObject *nearestHit = NULL;
  HitInfo nearestHitInfo, objHitInfo;
  nearestHitInfo.t = objHitInfo.t = RT_FLT_MAX;

  for ( size_t i=0; i < sceneObjects.size(); i++ ) {
    SceneObject *obj = sceneObjects[i];

    // reset texture field in HitInfo (this field is typically contains
    // the index of a triangle in a mesh)
    objHitInfo.texInfo = -1;

    // does ray intersect this object?
    if ( obj->intersectRay(ray, objHitInfo) ) {
      // is this the closest hit?
      if ( objHitInfo.t < nearestHitInfo.t && objHitInfo.t > 0.0f ) {
        nearestHitInfo = objHitInfo;
        nearestHit = obj;
      }
    }
  }

  // return false if there was no intersection
  if ( NULL == nearestHit )
    return NULL;

  hitInfoOut = nearestHitInfo;

  return nearestHit;
} // ListScene::findNearestHit()


bool ListScene::occlusionTest(const Ray &ray, float maxT) const {
  for ( size_t i=0; i < sceneObjects.size(); i++ ) {
    // does ray intersect this object?
    if ( sceneObjects[i]->occlusionTest(ray, maxT) )
      return true; // occluded
  }

  return false; // not occluded
} // ListScene::occlusionTest()
