/***************************************************************************
 *   Copyright (C) 2007 by A.J. Tavakoli                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef POINT2_H
#define POINT2_H

#include <iostream>

namespace Math3D {


template<class Real>
class Point2 {
public:
  // components of vector
  Real m[2];

  inline Point2() { }
  inline Point2(const Point2 &v) { m[0] = v.m[0]; m[1] = v.m[1]; }
  inline Point2(Real x, Real y) { m[0] = x; m[1] = y; }
  inline Point2(const Real *v) { m[0] = v[0]; m[1] = v[1]; }

   // overloaded operators
  inline Real & operator [] (int i);
  inline const Real & operator [] (int i) const;

  inline void set(Real _x, Real _y);
};


template<class Real>
inline Real & Point2<Real>::operator [] (int i) {
  // no error checking done for the sake of speed
  return m[i];
}


template<class Real>
inline const Real & Point2<Real>::operator [] (int i) const {
  // no error checking done for the sake of speed
  return m[i];
}


// assign the values of the vector's components to x, y, and z
template<class Real>
inline void Point2<Real>::set(Real x, Real y) {
  m[0] = x;
  m[1] = y;
}


// overloaded stream insertion operator for Point2
template<class Real>
std::ostream& operator << (std::ostream &out, const Point2<Real> &p) {
  out << "<" << p.m[0] << ", " << p.m[1] << ">";
  return out;
}

} // namespace Math3D


#endif
