
#ifndef QUAT_H
#define QUAT_H

#include "numeric.h"
#include "Vector3.h"
#include "Matrix3.h"
#include "Matrix4.h"

namespace Math3D {

template<class Real>
class Quat {
public:
   Real c[4];

   Quat();
   Quat(Real x, Real y, Real z, Real w);

   // overloaded operators for quaternion multiplication
   Quat operator *(const Quat &q) const;
   const Quat & operator *= (const Quat &q);

   inline Quat conjugate() const { return Quat(-c[0], -c[1], -c[2], c[3]); }
   Quat inverse() const;
   Real length() const;
   void normalize();
   Quat pow(Real t);
   void toMatrix(Real mat[]) const;
   Matrix3<Real> toMatrix3() const;
   Matrix4<Real> toMatrix4() const;

   static Quat fromAxisAngle(const Vector3<Real>& v, Real angle);
   static Quat fromEulerAngles(Real rotX, Real rotY, Real rotZ);

   inline Real& operator [] (int i)       { return c[i]; }
   inline Real  operator [] (int i) const { return c[i]; }
}; // Quat

template<class Real>
Quat<Real> slerp(const Quat<Real> &q1, const Quat<Real> &q2, Real t);

} // namespace Math3D

#include "Quat.cpp"

#endif
