
#ifndef RAY_H
#define RAY_H

#include <cmath>
#include "Vector3.h"
#include "Point3.h"


namespace Math3D {


template<class Real>
class Ray {
public:
  inline Ray();
  inline Ray(const Point3<Real>& o, const Vector3<Real>& d);

  inline void setOrigin(const Point3<Real>& o) { origin = o; }
  inline void setDir(const Vector3<Real>& d);
  inline void setDirDontNormalize(const Vector3<Real> &d);
  inline const Point3<Real>&  getOrigin() const { return origin; }
  inline const Vector3<Real>& getDir() const { return dir; }
private:
  Point3<Real> origin;
  Vector3<Real> dir;
};


template<class Real>
inline Ray<Real>::Ray() {

}


template<class Real>
inline Ray<Real>::Ray(const Point3<Real> &o, const Vector3<Real>& d):
    origin(o),
    dir(d) {
  dir.normalize();
}


template<class Real>
inline void Ray<Real>::setDir(const Vector3<Real>& d) {
  dir = d;
  dir.normalize();
}


template<class Real>
inline void Ray<Real>::setDirDontNormalize(const Vector3<Real> &d) {
  dir = d;
}

}

#endif
