
#ifndef RAYTRACER_H
#define RAYTRACER_H

#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>
#include <string>
#include <vector>
#include "Scene.h"
#include "Camera.h"


class RayTracer {
public:
  RayTracer(int w, int h, int nThreads, Scene *_scene);
  virtual ~RayTracer();

  virtual void mainLoop();
  bool loadScene(char *filename);
  virtual void render(int x, int y, int w, int h);
  
  void updateRayDirs();

  inline Camera& getCamera() { return camera; }

  inline int getImageWidth()  const { return imageWidth;  }
  inline int getImageHeight() const { return imageHeight; }

  inline const Uint8 *getImage() const { return imageData; }
protected:
  RGBColor<float> traceRay(const Ray &ray,
                           float refrIndex=1.0,
                           int depth=0,
                           int maxDepth=2);
  Uint8 *imageData;
  int imageWidth,
      imageHeight;
  Scene *scene;
  Camera camera;

private:
  class RenderThreadArgs {
  public:
    RayTracer *rayTracer;
    int x;
    int y;
    int w;
    int h;
  };

  class DynamicRegion {
  public:
    int x0, y0, x1, y1;
  };

  // renderThread() and renderThreadDynamicRegions() are the thread functions
  // for the worker threads for fixed regions and dynamic regions, respectively
  static int renderThread(void *data);
  static int renderThreadDynamicRegions(void *rayTracerP);

  // renderInternal() does the actual ray-casting for a thread
  void renderInternal(int x, int y, int w, int h);
  
  // same functionality as renderInternal(), but uses SSE packet tracing
  void renderInternalPktTrace(int x, int y, int w, int h);

  // when using dynamic regions, getNextDynamicRegion() returns the next
  // region that has not been assigned to a thread
  const DynamicRegion* getNextDynamicRegion();

  int numThreads;
  int numThreadWorkTiles;
  bool usingFixedRegions;
  int currentDynamicRegion;
  SDL_mutex *dynamicRegionLock;
  SDL_Thread **threads;
  DynamicRegion *dynamicRegions;
  Vector3 *rayDirs,
          *rotatedRayDirs;

  // parameters for perspective
  float viewDistance;
}; // RayTracer


#endif
