
#ifndef SCENE_H
#define SCENE_H

#include "common.h"
#include "SceneObject.h"
#include "Light.h"
#include "Material.h"
#include "TriangleMesh.h"
#include "SkyBox.h"


class Scene {
public:
  Scene();
  virtual ~Scene();

  virtual void insertObject(SceneObject *) = 0;

  virtual void insertLight(const Light &light) = 0;

  virtual SceneObject *findNearestHit(const Ray &ray,
                                      HitInfo &hitInfo) = 0;
                                      
  virtual bool occlusionTest(const Ray &ray, float maxT) const = 0;

  virtual const std::vector<Light>& getLights() const
     { throw RTException("Scene::getLights() not implemented"); } // TODO: get rid of this

  virtual RGBColor<float> lightContrib(const Ray &ray,
                                       const Point3 &pos,
                                       const Vector3 &n,
                                       int matID,
                                       const Point2 &tc) = 0;

  virtual void build() { } // not all scene types have a build method

  void clearScene();
  void loadXMLScene(const char *filename);

  TriangleMesh *loadModel(const char *filename, int matID, float scale=1.0f);
  Texture2D *loadTexture(const char *filename);
  void loadSkyBox(const std::string textureNames[6]);
  
  void insertMaterial(const Material &material);

  inline const Material *getMaterials() const { return &materials[0]; }
  inline const SkyBox   *getSkyBox()    const { return skyBox;        }
protected:
  RGBColor<float> singleLightContrib(const Light &light,
                                     const Point3 &p,
                                     const Ray &ray,
                                     const Vector3 &n,
                                     const Material &material);

  std::vector<Material> materials;

private:
  std::vector<Texture2D *> textures;
  std::vector<TriangleMesh *> meshes;
  
  SkyBox *skyBox;
};


#endif
