
#ifndef SCENEOBJECT_H
#define SCENEOBJECT_H

#include "common.h"


// interface to any type of object that can be inserted
// into a scene
class SceneObject {
public:
  virtual ~SceneObject() { }

  // indicates whether ray hits object and stores some information about
  // intersection with object in hitInfo (if there is an intersection)
  virtual bool intersectRay(const Ray &ray,
                            HitInfo &hitInfo) const = 0;
  
  // returns true if ray hits object and parametric distance to hit
  // is less than maxT (used for shadow rays)
  virtual bool occlusionTest(const Ray &ray,
                             float maxT) const = 0;

  virtual int getMatID() const = 0;
                        
  virtual void getNormalAndTCAt(const Point3 &p,
                                int texInfo,
                                const Material &mat,
                                Point2 &tc,
                                Vector3 &normal) const = 0;

  // probably not a good idea to call getAABB() in real-time since subclasses
  // might calculate AABB each time it is called
  virtual AABB getAABB() const = 0;
};


#endif
