
#include "SkyBox.h"
#include "TriangleMesh.h"


SkyBox::SkyBox(const Texture2D *const _textures[6]) {
  for ( int i=0; i < 6; i++ )
    textures[i] = _textures[i];       
}


void SkyBox::intersectRay(const Ray &ray, RGBColor<float> &color) const {
  static const int uvAxis[6][2] = { {2, 1}, {2, 1},
                                    {0, 1}, {0, 1},
                                    {0, 2}, {0, 2} };
  static const int flipU = 0x35;
  static const int flipV = 0x1F;
  
  
  //const Point3  &origin = ray.getOrigin();
  const Vector3 &dir = ray.getDir();

  float t1    = 0.0,
        t2    = 0.0,
        tNear = -RT_FLT_MAX;
        
  // faces: 0 - right, 1 - left, 2 - front, 3 - back, 4 - top, 5 - bottom
  int face = 0;

  // test against planes in x-axis if ray is not parallel to them
  if ( dir.m[0] >= 0.0001f || dir.m[0]  <= -0.0001f ) {
    // compute intersection distance of planes
    t1 = -1.0f/dir.m[0];
    t2 = 1.0f/dir.m[0];

    // t1 should be nearest intersection
    if ( t1 > t2 ) {
      t1 = t2;
      face = 1; // left
    }
    
    tNear = t1;
  }

  // test y planes if ray is not pallel to them
  if ( dir.m[1] >= 0.0001f || dir.m[1] <= -0.0001f ) {
    // compute intersection distance of planes
    t1 = -1.0f/dir.m[1];
    t2 = 1.0f/dir.m[1];

    // t1 should be nearest intersection
    if ( t1 > t2 ) {
      t1 = t2;
      
      // we want largest tNear
      if ( t1 > tNear ) {
        tNear = t1;
        face = 5; // bottom
      }
    }
    else if ( t1 > tNear ) {
      tNear = t1;
      face = 4; // top
    }
  }

  // test z planes if we are not parallel to them
  if ( dir.m[2] >= 0.0001f || dir.m[2] <= -0.0001f ) {
    // compute intersection distance of planes
    t1 = -1.0f/dir.m[2];
    t2 = 1.0f/dir.m[2];

    // t1 should be nearest intersection
    if (t1 > t2) {
      t1 = t2;

      // we want largest tNear
      if (t1 > tNear) {
        tNear = t1;
        face = 2; // front
      }
    }
    else if (t1 > tNear) {
      tNear = t1;
      face = 3; // back
    }
  } 
  
  // calculate intersection with skybox
  Vector3 p = dir*tNear;
  
  float u = (1.0f + p.m[ uvAxis[face][0] ])*0.5f,
        v = (1.0f + p.m[ uvAxis[face][1] ])*0.5f;
  
  if ( flipU & (1 << face) )
    u = 1.0f - u;
  if ( flipV & (1 << face) )
    v = 1.0f - v;

  // sample from texture
  if ( textures[face] )
    color = textures[face]->sample(u, v);
    //color = textures[face]->sampleBilinear(u, v);
}
