
#include <cmath>
#include "SphereObject.h"
#include "Math3D/intersect.h"


SphereObject::SphereObject(const Point3 &p, float r, int mat):
     pos(p),
     radius(r),
     matID(mat) {

}


bool SphereObject::intersectRay(const Ray &ray,
                                HitInfo &hitInfo) const {
  bool result = Math3D::intersectRaySphere<float>(ray, pos, radius, &hitInfo.t, NULL);

  if ( false == result || hitInfo.t < 0.0 )
    return false;

  return true;
}


bool SphereObject::occlusionTest(const Ray &ray,
                                 float maxT) const {
  float t = RT_FLT_MAX;
  bool result = Math3D::intersectRaySphere<float>(ray, pos, radius, &t, NULL);

  if ( false == result ||
       t < 0.0         ||
       t > maxT )
    return false; // ray does not hit sphere within
                  // distance given by maxT

  return true; // hit within given distance
}


void SphereObject::getNormalAndTCAt(const Point3 &p,
                                    int texInfo,
                                    const Material &mat,
                                    Point2 &tc,
                                    Vector3 &normal) const {
  // set texture coordinates
  Vector3 d = p - pos;
  
  tc[1] = Math3D::racos( (d[2]/radius)/RT_PI );

  if ( p[1] >= 0.0 )
    tc[0] = Math3D::racos(d[0]/(radius*Math3D::rsin(RT_PI*tc[1])))/RT_PI_MUL_2;
  else
    tc[0] = (RT_PI + Math3D::racos(d[0]/(radius*sin(tc[1]*RT_PI))))/RT_PI_MUL_2;
    
  // set normal
  normal = d;
  normal.normalize();
}


// returns AABB containing sphere
AABB SphereObject::getAABB() const {  
  float len = radius*1.73205081f;
  return AABB( pos, Vector3(len, len, len) );
}
