
#ifndef KDTREESCENE_H
#define KDTREESCENE_H

#include "TriangleMesh.h"


class TriangleMeshKDTree : public TriangleMesh {
public:
  TriangleMeshKDTree(int _matID, const char *filename, float scale=1.0f);
  virtual ~TriangleMeshKDTree();

  virtual bool intersectRay(const Ray &ray, HitInfo &hitInfo) const;
                            
  virtual bool occlusionTest(const Ray &ray, float maxT) const;

  class BoxEdge {
  public:
    BoxEdge() { }
    BoxEdge(float _pos, int _triIndex, bool _start);
    bool operator < (const BoxEdge &e) const;
    float pos;
    int triIndex;
    bool start;
  };

  class KDTreeNode {
  public:
    KDTreeNode(const AABB &nodeAABB,
               const Point3 *triVerts,
               const int *triIndices,
               int maxLeafTris,
               const std::vector<int> &trisInNode,
               const std::vector<AABB> &triAABBs,
               int badRefines,
               int depth);
    virtual ~KDTreeNode();

    struct InternalNodeData {
      float planeOffset;
      KDTreeNode *left,
                 *right;
    };

    struct LeafNodeData {
      int numTris;
      int *tris;
    };

    inline bool isLeaf() { return attrs == 0x03; }

    inline       unsigned char     getAttrs()            const { return attrs;            }
    inline const InternalNodeData &getInternalNodeData() const { return internalNodeData; }
    inline const LeafNodeData     &getLeafNodeData()     const { return leafNodeData;     }
  private:        
    void createLeafNode(const std::vector<int> &trisInNode);
    void createInternalNode(const AABB &nodeAABB,
                            const Point3 *triVerts,
                            const int *triIndices,
                            const std::vector<int> &trisInNode,
                            const std::vector<AABB> &triAABBs,
                            int maxLeafTris,
                            int badRefines,
                            int depth);
    // TODO: make this smaller (preferably less than 8 bytes)
    // flags: bits 0-2 - axis of split (for internal nodes), both are set
    //                   for leaf nodes
    unsigned char attrs;

    union {
      InternalNodeData internalNodeData; // offset of split and child node pointers (for internal nodes)
      LeafNodeData leafNodeData;         // object data (for leaf nodes)
    };
  };

  static const float intersectionCost;
  static const float traversalCost;
  static const float emptyBonus;
  
  static const int maxTrisInLeaf;
  static const int maxBadRefines;

private:
  void buildKDTree();

  KDTreeNode *root;
};


#endif
