
#ifndef TRIANGLEMESHOBJECT_H
#define TRIANGLEMESHOBJECT_H


#include "SceneObject.h"
#include "TriangleMesh.h"


class TriangleMeshObject : public SceneObject {
public:
  TriangleMeshObject(const TriangleMesh &_mesh,
                     int _matID,
                     const Point3 &_pos,
                     float rotX=0.0f,
                     float rotY=0.0f,
                     float rotZ=0.0f);
  virtual ~TriangleMeshObject();
  
  virtual int getMatID() const { return matID; }
                       
  virtual void getNormalAndTCAt(const Point3 &p,
                                int texInfo,
                                const Material &mat,
                                Point2 &tc,
                                Vector3 &normal) const;

  bool intersectRay(const Ray &ray, HitInfo &hitInfo) const;
  
  virtual bool occlusionTest(const Ray &ray, float maxT) const;

  void setPos(const Point3 &p);
  void setOrientation(float rotX, float rotY, float rotZ);
  virtual AABB getAABB() const;
protected:
  const TriangleMesh &mesh; // triangle mesh data
  int matID;                // material ID
  Point3 pos;               // position of object
  AABB aabb;                // bounding box of object
  Quat orient;              // orientation of object
  Matrix3 invRotMat;        // inverse rotation matrix (for rotating ray)
}; // TriangleMesh

#endif
