
#ifndef COMMON_H
#define COMMON_H

#include <cstddef>
#include <cstdlib>
#include <cstdio>
#include <stdexcept>
#include <climits>
#include <limits>
#include <vector>
#include <string>
#include <SDL/SDL.h>
#include "RGBColor.h"
#include "Material.h"
#include "Math3D/Vector3.h"
#include "Math3D/Point2.h"
#include "Math3D/Point3.h"
#include "Math3D/Plane.h"
#include "Math3D/Ray.h"
#include "Math3D/Matrix3.h"
#include "Math3D/Quat.h"
#include "Math3D/numeric.h"

#ifndef FLT_MAX
#define RT_FLT_MAX   (std::numeric_limits<float>::max())
#else
#define RT_FLT_MAX   FLT_MAX
#endif

#define RT_PI                 3.14159265f
#define RT_PI_MUL_2           6.2831853f

#define RT_EPSILON            0.0078125f


// typedefs for templated types for math library
typedef Math3D::Vector3<float> Vector3;
typedef Math3D::Point2<float>  Point2;
typedef Math3D::Point3<float>  Point3;
typedef Math3D::Plane<float>   Plane;
typedef Math3D::Ray<float>     Ray;
typedef Math3D::Matrix3<float> Matrix3;
typedef Math3D::AABB<float>    AABB;
typedef Math3D::Quat<float>    Quat;


// represents portion of image to be rendered
// by client
struct ImageRect {
  ImageRect(): x(0), y(0), width(0), height(0) { }
  ImageRect(Uint32 _x, Uint32 _y, Uint32 w, Uint32 h):
    x(_x), y(_y), width(w), height(h) { }

  Uint32 x;
  Uint32 y;
  Uint32 width;
  Uint32 height;
}; // ScreenRect


class RTException : public std::runtime_error {
public:
  RTException(const std::string &msg): std::runtime_error(msg)
    { /*std::cerr << msg << std::endl;*/ }
};


class HitInfo {
public:
  inline HitInfo() {
    //u = 0.0f;
    //v = 0.0f;
    texInfo = -1;
  }

  float t;        // distance to intersection point from ray origin
  //Vector3 normal; // normal at hit point
  int texInfo;    // this is optionally set by the intersectRay() method of
                  // an object

  //float u, v;     // texture coordinates at hitpoint
};


#endif
